
module sample_top (
//--------------Clocks------------------
	input          clkin_50,             //2.5V    //Bank 8B  
	input    		clkin_100m_p_0,         //LVDS    // 
	output			CLKOUT_SMA,

//--------------User-IO-----------------
	input          cpu_resetn,           //1.8V    //TR=0
	output [5:0]   user_led,             //1.8V
	input  [3:0]   user_pb,              //1.8V    //TR=0
	input	 [7:0]	user_dipsw,

	output	[6:0] 	hsma_tx_d,       //LVDS      //
	input		[6:0] 	hsma_rx_d,       //LVDS      //
	output	[16:7]	hsma_tx_d_p,       //LVDS      //
	input		[16:7]	hsma_rx_d_p,       //LVDS      //
	input		[2:1]		hsma_clk_in_p,     //LVDS
	output	[2:1]		hsma_clk_out_p,    //LVDS

	inout		[3:0]		hsma_d,            //2.5V      //Bank
	input					hsma_sda,          //Make input for loopback test
	output				hsma_scl,          //2.5V      //TR=0	
	
	output [34:0]		rx0_pfn,
	output [34:0]		rx1_pfn,
	output [7:0]		rx2_pfn
);

//------------------------------------------------------------------------------
// POLY_LENGHT POLY_TAP INV_PATTERN  || nbr of   bit seq.   max 0      feedback   
//                                   || stages    length  sequence      stages  
//------------------------------------------------------------------------------ 
//     7          6       false      ||    7         127      6 ni        6, 7   
//     9          5       false      ||    9         511      8 ni        5, 9   
//    11          9       false      ||   11        2047     10 ni        9,11   
//    15         14       true       ||   15       32767     15 i        14,15   
//    20          3       false      ||   20     1048575     19 ni        3,20   
//    23         18       true       ||   23     8388607     23 i        18,23   
//    29         27       true       ||   29   536870911     29 i        27,29   
//    31         28       true       ||   31  2147483647     31 i        28,31   
//------------------------------------------------------------------------------
parameter data_width		= 35;
parameter INV_PATTERN	= 0;
parameter POLY_LENGHT	= 7;
parameter POLY_TAP		= 6;

parameter cmos_width		= 7;
	
wire [data_width-1:0]	tx0_in;
wire [data_width-1:0]	tx1_in;
wire [2:0]	aln_done;
wire [1:0]	tx_coreclock;
wire [1:0]	tx_locked;
wire [1:0]	rx_locked;

wire [data_width-1:0]	rx0_out;
wire [data_width-1:0]	rx1_out;
wire [1:0]  rx_data_align;
wire [1:0]	rx_outclock;

wire clk_100m;
wire clk_20m;
wire sys_pll_locked;

wire [2:0]	error_sig;
reg  [2:0]	error_flag;

wire [7:0]	c_mos_comp;
wire [7:0]	c_mos_comp_inv;


assign user_led[0]	= !sys_pll_locked;
assign user_led[1]	= !tx_locked[0] | !tx_locked[1];
assign user_led[2]	= !rx_locked[0] | !rx_locked[1];
assign user_led[3]	= !aln_done[0]	 | !aln_done[1];
assign user_led[4]	= error_flag[0];
assign user_led[5]	= error_flag[1];

		

assign c_mos_comp_inv = ~c_mos_comp; 

//------------------------------------------------//
//******* USER PLL *******************************//
//------------------------------------------------//
sys_pll	sys_pll_inst (
	.inclk0 		( clkin_100m_p_0 ),
	.c0 			( clk_100m ),
	.c1 			( clk_20m  ),
	.locked 		( sys_pll_locked )
	);


	
//------------------------------------------------//
//******* LVDS LINK A TX *************************//
//------------------------------------------------//
	
lvds_tx	lvds_tx_inst0 (
	.pll_areset 		( !cpu_resetn | !sys_pll_locked ),
	.tx_in 				( tx0_in ),
	.tx_inclock 		( clk_100m ),
	.tx_coreclock 		( tx_coreclock[0] ),
	.tx_locked 			( tx_locked[0] ),
	.tx_out 				( hsma_tx_d_p[11:7] ),
	.tx_outclock 		( hsma_clk_out_p[1] )
);

test_pattern_gen test_pattern_gen_inst0
(
	.clk					(clk_100m) ,	// input  clk_sig
	.reset_n				(tx_locked[0]) ,		// input  reset_n_sig
	.err_inj				({data_width{!user_pb[0]}}) ,						// input  err_inj_sig
	.aln_done			(aln_done[0]) ,		// input  aln_done_sig
	.data_out			(tx0_in) 				// output [data_width-1:0] data_out_sig
);
defparam test_pattern_gen_inst0.data_width 	= data_width;
defparam test_pattern_gen_inst0.INV_PATTERN 	= INV_PATTERN;
defparam test_pattern_gen_inst0.POLY_LENGHT 	= POLY_LENGHT;
defparam test_pattern_gen_inst0.POLY_TAP 		= POLY_TAP;



//------------------------------------------------//
//******* LVDS LINK A RX *************************//
//------------------------------------------------//

lvds_rx	lvds_rx_inst0 (
	.pll_areset 		( !cpu_resetn ),
	.rx_data_align 	( rx_data_align[0] ),
	.rx_in 				( hsma_rx_d_p[11:7] ),
	.rx_inclock 		( hsma_clk_in_p[1] ),
	.rx_locked 			( rx_locked[0] ),
	.rx_out 				( rx0_out ),
	.rx_outclock 		( rx_outclock[0] )
	);
	
test_pattern_chk test_pattern_chk_inst0
(
	.clk					(rx_outclock[0]) ,	// input  clk_sig
	.reset_n				(rx_locked[0]) ,		// input  reset_n_sig
	.aln_done			(aln_done[0]) ,		// output  aln_done_sig
	.rx_d_aln			(rx_data_align[0]) ,	// output  rx_d_aln_sig
	.data_in				(rx0_out) ,				// input [data_width-1:0] data_in_sig
	.pfn					(rx0_pfn) 					// output [data_width-1:0] pfn_sig
);
defparam test_pattern_chk_inst0.data_width  = data_width;
defparam test_pattern_chk_inst0.INV_PATTERN = INV_PATTERN;
defparam test_pattern_chk_inst0.POLY_LENGHT = POLY_LENGHT;
defparam test_pattern_chk_inst0.POLY_TAP 	 = POLY_TAP;

assign error_sig[0] = |rx0_pfn;

always@(posedge error_sig[0] or negedge cpu_resetn)
begin
	if(!cpu_resetn)
	begin                        

		error_flag[0]	<= 0;
	end
	else 
	begin
		error_flag[0]	<= 1;
	end
end



//------------------------------------------------//
//******* LVDS LINK B TX *************************//
//------------------------------------------------//

lvds_tx	lvds_tx_inst1 (
	.pll_areset 		( !cpu_resetn | !sys_pll_locked ),
	.tx_in 				( tx1_in ),
	.tx_inclock 		( clk_100m ),
	.tx_coreclock 		( tx_coreclock[1] ),
	.tx_locked 			( tx_locked[1] ),
	.tx_out 				( hsma_tx_d_p[16:12] ),
	.tx_outclock 		( hsma_clk_out_p[2] )
);

test_pattern_gen test_pattern_gen_inst1
(
	.clk					(clk_100m) ,	// input  clk_sig
	.reset_n				(tx_locked[1]) ,		// input  reset_n_sig
	.err_inj				({data_width{!user_pb[0]}}) ,						// input  err_inj_sig
	.aln_done			(aln_done[1]) ,		// input  aln_done_sig
	.data_out			(tx1_in) 				// output [data_width-1:0] data_out_sig
);
defparam test_pattern_gen_inst1.data_width 	= data_width;
defparam test_pattern_gen_inst1.INV_PATTERN 	= INV_PATTERN;
defparam test_pattern_gen_inst1.POLY_LENGHT 	= POLY_LENGHT;
defparam test_pattern_gen_inst1.POLY_TAP 		= POLY_TAP;



//------------------------------------------------//
//******* LVDS LINK B RX *************************//
//------------------------------------------------//

lvds_rx	lvds_rx_inst1 (
	.pll_areset 		( !cpu_resetn ),
	.rx_data_align 	( rx_data_align[1] ),
	.rx_in 				( hsma_rx_d_p[16:12] ),
	.rx_inclock 		( hsma_clk_in_p[1] ),
	.rx_locked 			( rx_locked[1] ),
	.rx_out 				( rx1_out ),
	.rx_outclock 		( rx_outclock[1] )
	);
	
test_pattern_chk test_pattern_chk_inst1
(
	.clk					(rx_outclock[1]) ,	// input  clk_sig
	.reset_n				(rx_locked[1]) ,		// input  reset_n_sig
	.aln_done			(aln_done[1]) ,		// output  aln_done_sig
	.rx_d_aln			(rx_data_align[1]) ,	// output  rx_d_aln_sig
	.data_in				(rx1_out) ,				// input [data_width-1:0] data_in_sig
	.pfn					(rx1_pfn) 					// output [data_width-1:0] pfn_sig
);
defparam test_pattern_chk_inst1.data_width  = data_width;
defparam test_pattern_chk_inst1.INV_PATTERN = INV_PATTERN;
defparam test_pattern_chk_inst1.POLY_LENGHT = POLY_LENGHT;
defparam test_pattern_chk_inst1.POLY_TAP 	  = POLY_TAP;

assign error_sig[1] = |rx1_pfn;


always@(posedge error_sig[1] or negedge cpu_resetn)
begin 
	if(!cpu_resetn)
	begin                                 
		error_flag[1]	<= 0;
	end
	else 
	begin
		error_flag[1]	<= 1;
	end
end



//------------------------------------------------//
//******* C-MOS **********************************//
//------------------------------------------------//


assign hsma_scl			= user_pb[2];
assign hsma_d[0]			= user_pb[2];
assign hsma_d[2]			= user_pb[2];
assign hsma_tx_d[4:0] 	= {5{user_pb[2]}};
assign c_mos_comp[0]		= hsma_sda;
assign c_mos_comp[1]		= hsma_d[1];
assign c_mos_comp[2]		= hsma_d[3];
assign c_mos_comp[7:3]	= hsma_rx_d[4:0];



endmodule
